@@ -1,17 +1,16 @@
-from typing import Tuple, List, Union, Any
+from typing import Any, Dict, Tuple, List, Set, Union
 import warnings
 
-from fastapi import HTTPException
 from tqdm import tqdm
 
 from aiida.orm.nodes import Node
 from aiida.orm.querybuilder import QueryBuilder
 
-from optimade.filterparser import LarkParser
 from optimade.models import EntryResource
-from optimade.server.config import CONFIG
-from optimade.server.exceptions import BadRequest, Forbidden
+from optimade.server.entry_collections import EntryCollection
+from optimade.server.exceptions import BadRequest, NotFound
 from optimade.server.query_params import EntryListingQueryParams, SingleEntryQueryParams
+from optimade.server.warnings import UnknownProviderProperty
 
 from aiida_optimade.common import CausationError
 from aiida_optimade.common.logger import LOGGER
@@ -20,7 +19,7 @@ from aiida_optimade.transformers import AiidaTransformer
 from aiida_optimade.utils import retrieve_queryable_properties
 
 
-class AiidaCollection:
+class AiidaCollection(EntryCollection):
     """Collection of AiiDA entities"""
 
     CAST_MAPPING = {
@@ -37,147 +36,29 @@ class AiidaCollection:
         resource_cls: EntryResource,
         resource_mapper: ResourceMapper,
     ):
-        self.entities = entities if isinstance(entities, list) else [entities]
-        self.parser = LarkParser()
-        self.resource_cls = resource_cls
-        self.resource_mapper = resource_mapper
+        super().__init__(
+            resource_cls=resource_cls,
+            resource_mapper=resource_mapper,
+            transformer=AiidaTransformer(mapper=resource_mapper),
+        )
 
-        self.transformer = AiidaTransformer()
-        self.provider = CONFIG.provider.prefix
-        self.provider_fields = CONFIG.provider_fields.get(resource_mapper.ENDPOINT, [])
-        self.parser = LarkParser()
+        self.entities = entities if isinstance(entities, list) else [entities]
 
         # "Cache"
         self._data_available: int = None
         self._data_returned: int = None
-        self._filter_fields: set = None
-        self._latest_filter: dict = None
-        self._count: dict = None
+        self._extras_fields: Set[str] = None
+        self._latest_filter: Dict[str, Any] = None
+        self._count: Dict[str, Any] = None
         self._checked_extras_filter_fields: set = set()
 
-    def _clear_cache(self) -> None:
-        """Clear in-memory attributes cache"""
-        self._data_available: int = None
-        self._data_returned: int = None
-        self._filter_fields: set = None
-        self._latest_filter: dict = None
-        self._count: dict = None
-        self._checked_extras_filter_fields: set = set()
+        self._all_fields: Set[str] = None
 
-    def __len__(self) -> int:
-        """Get available data entries in collection (data_available)"""
-        return self.data_available
-
-    def get_attribute_fields(self) -> set:
-        """Get all attribute properties/fields for OPTIMADE entity"""
-        schema = self.resource_cls.schema()
-        attributes = schema["properties"]["attributes"]
-        if "allOf" in attributes:
-            allOf = attributes.pop("allOf")  # pylint: disable=invalid-name
-            for dict_ in allOf:
-                attributes.update(dict_)
-        if "$ref" in attributes:
-            path = attributes["$ref"].split("/")[1:]
-            attributes = schema.copy()
-            while path:
-                next_key = path.pop(0)
-                attributes = attributes[next_key]
-        return set(attributes["properties"].keys())
-
-    @staticmethod
-    def _find(node_types: List[str], **kwargs) -> QueryBuilder:
-        """Workhorse function to perform AiiDA QueryBuilder query"""
-        for key in kwargs:
-            if key not in {"filters", "order_by", "limit", "project", "offset"}:
-                raise ValueError(
-                    f"You supplied key {key!r}. _find() only takes the keys: "
-                    '"filters", "order_by", "limit", "project", "offset"'
-                )
-
-        filters = kwargs.get("filters", {})
-        filters["node_type"] = {"or": [{"==": node_type} for node_type in node_types]}
-        order_by = kwargs.get("order_by", None)
-        order_by = {Node: order_by} if order_by else {Node: {"id": "asc"}}
-        limit = kwargs.get("limit", None)
-        offset = kwargs.get("offset", None)
-        project = kwargs.get("project", [])
-
-        query = QueryBuilder(limit=limit, offset=offset)
-        query.append(Node, project=project, filters=filters)
-        query.order_by(order_by)
-
-        return query
-
-    def _find_all(self, **kwargs) -> list:
-        """Execute AiiDA QueryBuilder query, return all results."""
-        LOGGER.debug(
-            "Using QueryBuilder to get ALL projected values from found entries."
-        )
-        query = self._find(self.entities, **kwargs)
-        res = query.all()
-        del query
-        return res
-
-    def _perform_count(self, **kwargs) -> int:
-        """Instantiate new QueryBuilder object and perform count()"""
-        LOGGER.debug("Using QueryBuilder to COUNT all found entries.")
-        query = self._find(self.entities, **kwargs)
-        res = query.count()
-        del query
-        return res
-
-    def count(self, **kwargs) -> int:
-        """Count amount of data returned for query"""
-        LOGGER.debug("Calling count function in EntryCollection.")
-        if self._count is None:
-            LOGGER.debug("self._count is None")
-            self._count = {
-                "count": self._perform_count(**kwargs),
-                "filters": kwargs.get("filters", {}),
-                "limit": kwargs.get("limit", None),
-                "offset": kwargs.get("offset", None),
-            }
-        else:
-            for limiting_param in {"filters", "limit", "offset"}:
-                if kwargs.get(limiting_param, None) != self._count.get(
-                    limiting_param, None
-                ):
-                    if limiting_param == "filters":
-                        # As the `node_type` field is added to the filters in `_find()`,
-                        # this will make sure to check the _actual_ requested filter
-                        # fields.
-                        count_copy = self._count.get(limiting_param, {}).copy()
-                        kwargs_copy = kwargs.get(limiting_param, {}).copy()
-                        count_copy.pop("node_type", None)
-                        kwargs_copy.pop("node_type", None)
-                        if kwargs_copy == count_copy:
-                            continue
-                    elif limiting_param == "offset":
-                        if not kwargs.get(limiting_param, None) and not self._count.get(
-                            limiting_param, None
-                        ):
-                            # This will check also if offset in either is set to 0.
-                            # There is no difference if the default page_offset is
-                            # requested.
-                            continue
-                    LOGGER.debug(
-                        "%s was not the same as was found in self._count:\n"
-                        "self._count = %s\nkwargs = %s",
-                        limiting_param,
-                        self._count.get(limiting_param, None),
-                        kwargs.get(limiting_param, None),
-                    )
-                    self._count = {
-                        "count": self._perform_count(**kwargs),
-                        "filters": kwargs.get("filters", {}),
-                        "limit": kwargs.get("limit", None),
-                        "offset": kwargs.get("offset", None),
-                    }
-                    break
-            else:
-                LOGGER.debug("Using self._count")
-
-        return self._count.get("count", 0)
+    @property
+    def all_fields(self) -> Set[str]:
+        if not self._all_fields:
+            self._all_fields = super().all_fields
+        return self._all_fields
 
     @property
     def data_available(self) -> int:
@@ -192,7 +73,7 @@ class AiidaCollection:
         """Set _data_available if it has not yet been set"""
         if not self._data_available:
             LOGGER.debug("Setting data_available!")
-            self._data_available = self.count(project="id")
+            self._data_available = self.count()
 
     @property
     def data_returned(self) -> int:
@@ -220,31 +101,97 @@ class AiidaCollection:
             LOGGER.debug("Setting data_returned using filter: %s", self._latest_filter)
             self._data_returned = self.count(**criteria)
 
-    def find(
-        self, params: Union[EntryListingQueryParams, SingleEntryQueryParams]
-    ) -> Tuple[List[EntryResource], int, bool, set]:
-        """Find all requested AiiDA entities as OPTIMADE JSON objects"""
-        self.set_data_available()
-        criteria = self._parse_params(params)
+    def _clear_cache(self) -> None:
+        """Clear in-memory attributes cache"""
+        self._data_available: int = None
+        self._data_returned: int = None
+        self._extras_fields: set = None
+        self._latest_filter: dict = None
+        self._count: dict = None
+        self._checked_extras_filter_fields: set = set()
 
-        all_fields = criteria.pop("fields")
-        if getattr(params, "response_fields", False):
-            fields = set(params.response_fields.split(","))
-            fields |= self.resource_mapper.get_required_fields()
+    def __len__(self) -> int:
+        return self.data_available
+
+    def insert(self, _: List[EntryResource]) -> None:
+        raise NotImplementedError(
+            f"The insert method is not implemented for {self.__class__.__name__}."
+        )
+
+    def count(self, **kwargs) -> int:
+        LOGGER.debug("Calling count function in EntryCollection.")
+        if self._count is None:
+            LOGGER.debug("self._count is None")
+            self._count = {
+                "count": self._perform_count(**kwargs),
+                "filters": kwargs.get("filters", {}),
+                "limit": kwargs.get("limit", None),
+                "offset": kwargs.get("offset", None),
+            }
         else:
-            fields = all_fields.copy()
+            for limiting_param in {"filters", "limit", "offset"}:
+                if kwargs.get(limiting_param, None) != self._count.get(
+                    limiting_param, None
+                ):
+                    if limiting_param == "filters":
+                        # As the `node_type` field is added to the filters in
+                        # `_prepare_query()`, this will make sure to check the *actual*
+                        # requested filter fields.
+                        count_copy = self._count.get(limiting_param, {}).copy()
+                        kwargs_copy = kwargs.get(limiting_param, {}).copy()
+                        count_copy.pop("node_type", None)
+                        kwargs_copy.pop("node_type", None)
+                        if kwargs_copy == count_copy:
+                            continue
+                    elif limiting_param == "offset":
+                        if not kwargs.get(limiting_param, None) and not self._count.get(
+                            limiting_param, None
+                        ):
+                            # This will check also if offset in either is set to 0.
+                            # There is no difference if the default page_offset is
+                            # requested.
+                            continue
+                    LOGGER.debug(
+                        "%s was not the same as was found in self._count:\n"
+                        "self._count[%s] = %s\nkwargs[%s] = %s",
+                        limiting_param,
+                        limiting_param,
+                        self._count.get(limiting_param, None),
+                        limiting_param,
+                        kwargs.get(limiting_param, None),
+                    )
+                    self._count = {
+                        "count": self._perform_count(**kwargs),
+                        "filters": kwargs.get("filters", {}),
+                        "limit": kwargs.get("limit", None),
+                        "offset": kwargs.get("offset", None),
+                    }
+                    break
+            else:
+                LOGGER.debug("Using self._count")
 
-        if criteria.get("filters", {}) and self._get_extras_filter_fields():
-            for requested_extras_field in self._get_extras_filter_fields():
+        return self._count.get("count", 0)
+
+    def find(  # pylint: disable=too-many-branches
+        self, params: Union[EntryListingQueryParams, SingleEntryQueryParams]
+    ) -> Tuple[
+        Union[List[EntryResource], EntryResource, None], int, bool, Set[str], Set[str]
+    ]:
+        self.set_data_available()
+
+        criteria = self.handle_query_params(params)
+        single_entry = isinstance(params, SingleEntryQueryParams)
+        response_fields = criteria.pop("fields", set())
+
+        if criteria.get("filters", {}) and self._extras_fields:
+            for requested_extras_field in self._extras_fields:
                 if requested_extras_field not in self._checked_extras_filter_fields:
                     LOGGER.debug(
                         "Checking all extras fields have been calculated (and possibly "
                         "calculate them)."
                     )
                     self._check_and_calculate_entities()
-                    self._checked_extras_filter_fields |= (
-                        self._get_extras_filter_fields()
-                    )
+                    self._checked_extras_filter_fields |= self._extras_fields
                     break
             else:
                 LOGGER.debug(
@@ -258,159 +205,265 @@ class AiidaCollection:
 
         self.set_data_returned(**criteria)
 
-        entities = self._find_all(**criteria)
-        results = []
-        for entity in entities:
-            results.append(
-                self.resource_cls(
-                    **self.resource_mapper.map_back(
-                        dict(zip(criteria["project"], entity))
-                    )
-                )
-            )
+        results, more_data_available = self._run_db_query(
+            criteria=criteria, single_entry=single_entry
+        )
 
-        if isinstance(params, EntryListingQueryParams):
-            criteria_copy = criteria.copy()
-            criteria_copy.pop("limit", None)
-            offset = criteria_copy.pop("offset", 0)
-            more_data_available = len(results) < (self.count(**criteria_copy) - offset)
-        else:
-            more_data_available = False
+        if single_entry:
             if len(results) > 1:
-                raise HTTPException(
-                    status_code=404,
+                raise NotFound(
                     detail=f"Instead of a single entry, {len(results)} entries were "
                     "found",
                 )
 
-        if isinstance(params, SingleEntryQueryParams):
             results = results[0] if results else None
 
+        include_fields = (
+            response_fields - self.resource_mapper.TOP_LEVEL_NON_ATTRIBUTES_FIELDS
+        )
+
+        bad_optimade_fields = set()
+        bad_provider_fields = set()
+        for field in include_fields:
+            if field not in self.resource_mapper.ALL_ATTRIBUTES:
+                if field.startswith("_"):
+                    if any(
+                        field.startswith(f"_{prefix}_")
+                        for prefix in self.resource_mapper.SUPPORTED_PREFIXES
+                    ):
+                        bad_provider_fields.add(field)
+                else:
+                    bad_optimade_fields.add(field)
+
+        if bad_provider_fields:
+            warnings.warn(
+                UnknownProviderProperty(
+                    detail=(
+                        "Unrecognised field(s) for this provider requested in "
+                        f"`response_fields`: {bad_provider_fields}."
+                    )
+                )
+            )
+
+        if bad_optimade_fields:
+            raise BadRequest(
+                detail=(
+                    "Unrecognised OPTIMADE field(s) in requested `response_fields`: "
+                    f"{bad_optimade_fields}."
+                )
+            )
+
+        if results:
+            results = self.resource_mapper.deserialize(results)
+
         return (
             results,
             self.data_returned,
             more_data_available,
-            all_fields - fields,
+            self.all_fields - response_fields,
+            include_fields,
         )
 
-    def _alias_filter(self, filters: Any) -> Union[dict, list]:
-        """Get aliased field names in nested filter query.
+    def _run_db_query(
+        self, criteria: Dict[str, Any], single_entry: bool = False
+    ) -> Tuple[List[Dict[str, Any]], bool]:
+        """Run the query on the backend and collect the results.
+
+        Arguments:
+            criteria: A dictionary representation of the query parameters.
+            single_entry: Whether or not the caller is expecting a single entry
+                response.
+
+        Returns:
+            The list of entries from the database (without any re-mapping) and a
+            boolean for whether or not there is more data available.
 
-        I.e. turn OPTIMADE field names into AiiDA field names
         """
-        if isinstance(filters, dict):
-            res = {}
-            for key, value in filters.items():
-                new_value = value
-                if isinstance(value, (dict, list)):
-                    new_value = self._alias_filter(value)
-                aliased_key = self.resource_mapper.alias_for(key)
-                res[aliased_key] = new_value
-                self._filter_fields.add(aliased_key)
-        elif isinstance(filters, list):
-            res = []
-            for item in filters:
-                new_value = item
-                if isinstance(item, (dict, list)):
-                    new_value = self._alias_filter(item)
-                res.append(new_value)
+        results = []
+        for entity in self._find_all(**criteria):
+            results.append(dict(zip(criteria["project"], entity)))
+
+        if single_entry:
+            more_data_available = False
         else:
-            raise NotImplementedError(
-                "_alias_filter can only handle dict and list objects"
-            )
+            criteria_copy = criteria.copy()
+            criteria_copy.pop("limit", None)
+            offset = criteria_copy.pop("offset", 0)
+            more_data_available = len(results) < (self.count(**criteria_copy) - offset)
+
+        return results, more_data_available
+
+    @staticmethod
+    def _prepare_query(node_types: List[str], **kwargs) -> QueryBuilder:
+        """Workhorse function to prepare an AiiDA QueryBuilder query"""
+        for key in kwargs:
+            if key not in {"filters", "order_by", "limit", "project", "offset"}:
+                raise ValueError(
+                    f"You supplied key {key!r}. The only valid query keys are: "
+                    '"filters", "order_by", "limit", "project", "offset"'
+                )
+
+        filters = kwargs.get("filters", {})
+        filters["node_type"] = {"or": [{"==": node_type} for node_type in node_types]}
+        order_by = kwargs.get("order_by", None)
+        order_by = {Node: order_by} if order_by else {Node: {"id": "asc"}}
+        limit = kwargs.get("limit", None)
+        offset = kwargs.get("offset", None)
+        project = kwargs.get("project", [])
+
+        query = QueryBuilder(limit=limit, offset=offset)
+        query.append(Node, project=project, filters=filters)
+        query.order_by(order_by)
+
+        return query
+
+    def _find_all(self, **kwargs) -> list:
+        """Execute AiiDA QueryBuilder query, return all results."""
+        LOGGER.debug(
+            "Using QueryBuilder to get ALL projected values from found entries."
+        )
+        query = self._prepare_query(self.entities, **kwargs)
+        res = query.all()
+        del query
         return res
 
-    def _parse_params(self, params: EntryListingQueryParams) -> dict:
-        """Parse query parameters and transform them into AiiDA QueryBuilder concepts"""
-        cursor_kwargs = {}
+    def _perform_count(self, **kwargs) -> int:
+        """Instantiate new QueryBuilder object and perform count()"""
+        LOGGER.debug("Using QueryBuilder to COUNT all found entries.")
+        query = self._prepare_query(self.entities, **kwargs)
+        res = query.count()
+        del query
+        return res
+
+    def handle_query_params(
+        self, params: Union[EntryListingQueryParams, SingleEntryQueryParams]
+    ) -> Dict[str, Any]:
+        """Parse and interpret the backend-agnostic query parameter models into a
+        dictionary that can be used by AiiDA's QueryBuilder.
+
+        Parameters:
+            params: The initialized query parameter model from the server.
+
+        Raises:
+            Forbidden: If too large of a page limit is provided.
+            BadRequest: If an invalid request is made, e.g., with incorrect fields
+                or response format.
+
+        Returns:
+            A dictionary representation of the query parameters.
+
+        """
+        cursor_kwargs: dict = super().handle_query_params(params)
+
+        # Remove Mongo-specific fields introduced in the parent method
+        cursor_kwargs.get("projection", {}).pop("_id", None)
 
         # filter
-        if getattr(params, "filter", False):
-            aiida_filter = self.transformer.transform(self.parser.parse(params.filter))
-            self._filter_fields = set()
-            cursor_kwargs["filters"] = self._alias_filter(aiida_filter)
-
-        # response_format
-        if (
-            getattr(params, "response_format", False)
-            and params.response_format != "json"
-        ):
-            raise BadRequest(
-                detail=(
-                    f"Response format {params.response_format} is not supported, please"
-                    ' use response_format="json"'
-                )
-            )
-
-        # page_limit
-        if getattr(params, "page_limit", False):
-            limit = params.page_limit
-            if limit > CONFIG.page_limit_max:
-                raise Forbidden(
-                    detail=f"Max allowed page_limit is {CONFIG.page_limit_max}, "
-                    f"you requested {limit}",
-                )
-            cursor_kwargs["limit"] = limit
+        if cursor_kwargs.get("filter", False):
+            cursor_kwargs["filters"] = cursor_kwargs.pop("filter")
+            self._find_extras_fields(cursor_kwargs["filters"])
         else:
-            cursor_kwargs["limit"] = CONFIG.page_limit
+            cursor_kwargs.pop("filter", None)
 
         # response_fields
-        # All OPTIMADE fields
-        fields = self.resource_mapper.TOP_LEVEL_NON_ATTRIBUTES_FIELDS.copy()
-        fields |= self.get_attribute_fields()
-        # All provider-specific fields
-        fields |= {f"_{self.provider}_" + _ for _ in self.provider_fields}
-        cursor_kwargs["fields"] = fields
-        cursor_kwargs["project"] = list(
-            {self.resource_mapper.alias_for(f) for f in fields}
-        )
+        cursor_kwargs["project"] = list(cursor_kwargs.pop("projection", {}).keys())
 
         # sort
-        # NOTE: sorting only works for extras fields for the nodes already with
-        #       calculated extras. To calculate all extras, make a single filter query
-        #       using any extra field.
-        if getattr(params, "sort", False):
-            sort_spec = []
-            for entity_property in params.sort.split(","):
-                field = entity_property
-                sort_direction = "asc"
-                if entity_property.startswith("-"):
-                    field = field[1:]
-                    sort_direction = "desc"
-                aliased_field = self.resource_mapper.alias_for(field)
-
-                _, properties = retrieve_queryable_properties(
-                    self.resource_cls.schema(), {"id", "type", "attributes"}
-                )
-                field_type = properties[field].get(
-                    "format", properties[field].get("type", "")
-                )
-                if field_type == "array":
-                    raise TypeError("Cannot sort on a field with a list value type")
-
-                sort_spec.append(
-                    {
-                        aliased_field: {
-                            "order": sort_direction,
-                            "cast": self.CAST_MAPPING[field_type],
-                        }
-                    }
-                )
-            cursor_kwargs["order_by"] = sort_spec
+        if cursor_kwargs.get("sort", False):
+            cursor_kwargs["order_by"] = cursor_kwargs.pop("sort")
 
         # page_offset
-        if getattr(params, "page_offset", False):
-            cursor_kwargs["offset"] = params.page_offset
+        if cursor_kwargs.get("skip", False):
+            cursor_kwargs["offset"] = cursor_kwargs.pop("skip")
 
         return cursor_kwargs
 
-    def _get_extras_filter_fields(self) -> set:
-        """Get all queried fields saved in Node extras."""
-        return {
-            field[len(self.resource_mapper.PROJECT_PREFIX) :]  # noqa: E203
-            for field in self._filter_fields
-            if field.startswith(self.resource_mapper.PROJECT_PREFIX)
-        }
+    def parse_sort_params(self, sort_params: str) -> List[Dict[str, Dict[str, str]]]:
+        """Handles any sort parameters passed to the collection,
+        resolving aliases and dealing with any invalid fields.
+
+        Note:
+            Sorting only works for extras fields for the nodes already with calculated
+            extras. To calculate all extras, make a single filter query using any extra
+            field.
+
+        Raises:
+            BadRequest: If an invalid sort is requested.
+
+        Returns:
+            A list of dictionaries definining the sort order, type and indirectly the
+            internal priority.
+
+        """
+        sort_spec = []
+        for entity_property in sort_params.split(","):
+            field = entity_property
+            sort_direction = "asc"
+            if entity_property.startswith("-"):
+                field = field[1:]
+                sort_direction = "desc"
+            aliased_field = self.resource_mapper.get_backend_field(field)
+
+            _, properties = retrieve_queryable_properties(
+                self.resource_cls.schema(), {"id", "type", "attributes"}
+            )
+            field_type = properties[field].get(
+                "format", properties[field].get("type", "")
+            )
+            if field_type == "array":
+                raise TypeError("Cannot sort on a field with a list value type.")
+
+            sort_spec.append(
+                {
+                    aliased_field: {
+                        "order": sort_direction,
+                        "cast": self.CAST_MAPPING[field_type],
+                    }
+                }
+            )
+        return sort_spec
+
+    def _find_extras_fields(self, filters: Union[dict, list]) -> None:
+        """Collect all properties to be found in AiiDA Node extras.
+
+        Parameters:
+            filters: The complete or part of the parsed and transformed `filter` query
+                parameter.
+
+        """
+        from copy import deepcopy
+
+        def __filter_fields_util(  # pylint: disable=unused-private-member
+            _filters: Union[dict, list]
+        ) -> Union[dict, list]:
+            if isinstance(_filters, dict):
+                res = {}
+                for key, value in _filters.items():
+                    res[key] = (
+                        __filter_fields_util(value)
+                        if isinstance(value, (dict, list))
+                        else value
+                    )
+                self._extras_fields |= {
+                    key[len(self.resource_mapper.PROJECT_PREFIX) :]
+                    for key in _filters
+                    if key.startswith(self.resource_mapper.PROJECT_PREFIX)
+                }
+            elif isinstance(_filters, list):
+                res = [
+                    __filter_fields_util(item)
+                    if isinstance(item, (dict, list))
+                    else item
+                    for item in _filters
+                ]
+            else:
+                raise NotImplementedError(
+                    "_find_extras_fields can only handle dict and list objects."
+                )
+            return res
+
+        self._extras_fields = set()
+        __filter_fields_util(deepcopy(filters))
 
     def _check_and_calculate_entities(
         self, cli: bool = False, entries: List[List[int]] = None
@@ -422,6 +475,7 @@ class AiidaCollection:
 
         Parameters:
             cli: Whether or not this method is run through the CLI.
+            entries: AiiDA Node PKs.
 
         Returns:
             A list of the Node PKs representing the Nodes that were necessary to
@@ -429,28 +483,28 @@ class AiidaCollection:
 
         """
 
-        def _update_entities(entities: list, fields: list):
+        def _update_entities(entities: List[List[Any]], fields: List[str]):
             """Utility function to update entities within this method"""
-            optimade_fields = [self.resource_mapper.alias_of(_) for _ in fields]
+            optimade_fields = [
+                self.resource_mapper.get_optimade_field(_) for _ in fields
+            ]
             for entity in entities:
                 field_to_entity_value = dict(zip(optimade_fields, entity))
                 retrieved_attributes = field_to_entity_value.copy()
-                for missing_attribute in self._get_extras_filter_fields():
+                for missing_attribute in self._extras_fields:
                     retrieved_attributes.pop(missing_attribute)
                 self.resource_mapper.build_attributes(
                     retrieved_attributes=retrieved_attributes,
                     entry_pk=field_to_entity_value["id"],
                     node_type=field_to_entity_value["type"],
-                    missing_attributes=self._get_extras_filter_fields(),
+                    missing_attributes=self._extras_fields,
                 )
 
         extras_keys = [
             key for key in self.resource_mapper.PROJECT_PREFIX.split(".") if key
         ]
-        filter_fields = [
-            {"!has_key": field} for field in self._get_extras_filter_fields()
-        ]
-        necessary_entities_qb = (
+        filter_fields = [{"!has_key": field} for field in self._extras_fields]
+        necessary_entity_ids = (
             self._find_all(
                 filters={
                     "or": [
@@ -464,16 +518,18 @@ class AiidaCollection:
             else entries
         )
 
-        if necessary_entities_qb:
+        if necessary_entity_ids:
             # Necessary entities for the OPTIMADE query exist with unknown OPTIMADE
             # fields.
-            necessary_entity_ids = [pk[0] for pk in necessary_entities_qb]
+            necessary_entity_ids = [pk[0] for pk in necessary_entity_ids]
 
             # Create the missing OPTIMADE fields
             fields = {"id", "type"}
             fields |= self.get_attribute_fields()
-            fields |= {f"_{self.provider}_" + _ for _ in self.provider_fields}
-            fields = list({self.resource_mapper.alias_for(_) for _ in fields})
+            fields |= {
+                f"_{self.provider_prefix}_{field}" for field in self.provider_fields
+            }
+            fields = list({self.resource_mapper.get_backend_field(_) for _ in fields})
 
             entities = self._find_all(
                 filters={"id": {"in": necessary_entity_ids}}, project=fields
